from whattime import TimeType
from whattime import week_info


def test_time_type_state_is_weekday(monday, tuesday, wednesday, thursday, friday, saturday, sunday):
    """ Test week_info is_weekday of given datetime """

    assert week_info(monday).is_weekday is True
    assert week_info(tuesday).is_weekday is True
    assert week_info(wednesday).is_weekday is True
    assert week_info(thursday).is_weekday is True
    assert week_info(friday).is_weekday is True
    assert week_info(saturday).is_weekday is False
    assert week_info(sunday).is_weekday is False


def test_time_type_state_is_weekend(monday, tuesday, wednesday, thursday, friday, saturday, sunday):
    """ Test week_info is_weekday of given datetime """

    assert week_info(monday).is_weekend is False
    assert week_info(tuesday).is_weekend is False
    assert week_info(wednesday).is_weekend is False
    assert week_info(thursday).is_weekend is False
    assert week_info(friday).is_weekend is False
    assert week_info(saturday).is_weekend is True
    assert week_info(sunday).is_weekend is True


def test_time_type_state_certain_day(days, monday, tuesday, wednesday, thursday, friday, saturday,
                                     sunday):
    """ Test week_info days of given datetime """

    for day in days:
        assert week_info(day).is_monday is (day is monday)
        assert week_info(day).is_tuesday is (day is tuesday)
        assert week_info(day).is_wednesday is (day is wednesday)
        assert week_info(day).is_thursday is (day is thursday)
        assert week_info(day).is_friday is (day is friday)
        assert week_info(day).is_saturday is (day is saturday)
        assert week_info(day).is_sunday is (day is sunday)


def test_time_type_state_types(monday, tuesday, wednesday, thursday, friday, saturday, sunday):
    """ Test fitting types for the given datetime """

    assert week_info(monday).types == {TimeType.MONDAY, TimeType.WEEKDAY}
    assert week_info(tuesday).types == {TimeType.TUESDAY, TimeType.WEEKDAY}
    assert week_info(wednesday).types == {TimeType.WEDNESDAY, TimeType.WEEKDAY}
    assert week_info(thursday).types == {TimeType.THURSDAY, TimeType.WEEKDAY}
    assert week_info(friday).types == {TimeType.FRIDAY, TimeType.WEEKDAY}
    assert week_info(saturday).types == {TimeType.SATURDAY, TimeType.WEEKEND}
    assert week_info(sunday).types == {TimeType.SUNDAY, TimeType.WEEKEND}



from datetime import datetime

from whattime import whattime

# Asking for all types of time descriptors for a e.g. a monday afternoon:
now = datetime.now()
info = whattime(now)

print(info.types)
# {<TimeType.WEEKDAY: 'weekday'>, <TimeType.MONDAY: 'monday'>, ...}

print(info.is_weekday)
# True

print(info.is_weekend)
# False

print(info.is_monday)
# True

print(info.is_tuesday)
# True

print(info.is_afternoon)
# True

print(info.is_morning)
# False



# -*- coding: utf-8 -*-

from pkg_resources import get_distribution, DistributionNotFound

try:
    dist_name = __name__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = 'unknown'

__all__ = [
    'whattime',
    'TimeType',
    'week_info',
    'day_time_info'
]

from .utils import whattime, week_info, day_time_info
from .type import TimeType



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Dummy conftest.py for whattime.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""
from datetime import datetime, timedelta
from typing import List

import pytest


@pytest.fixture(scope='session')
def now() -> datetime:
    return datetime.utcnow()


@pytest.fixture
def today_noon(now: datetime) -> datetime:
    return now.replace(hour=12, minute=0, second=0, microsecond=0)


@pytest.fixture
def monday(today_noon: datetime) -> datetime:
    return today_noon - timedelta(days=today_noon.weekday())


@pytest.fixture
def tuesday(monday: datetime) -> datetime:
    return monday + timedelta(days=1)


@pytest.fixture
def wednesday(monday: datetime) -> datetime:
    return monday + timedelta(days=2)


@pytest.fixture
def thursday(monday: datetime) -> datetime:
    return monday + timedelta(days=3)


@pytest.fixture
def friday(monday: datetime) -> datetime:
    return monday + timedelta(days=4)


@pytest.fixture
def saturday(monday: datetime) -> datetime:
    return monday + timedelta(days=5)


@pytest.fixture
def sunday(monday: datetime) -> datetime:
    return monday + timedelta(days=6)


@pytest.fixture
def days(monday: datetime, tuesday: datetime, wednesday: datetime, thursday: datetime,
         friday: datetime, saturday: datetime, sunday: datetime) -> List[datetime]:
    return [monday, tuesday, wednesday, thursday, friday, saturday, sunday]



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Set

from .type import TimeType


class InfoBase(ABC):
    __mapping__ = {}

    def __init__(self, date: datetime):
        self.date = date
        self._types = set()

    @abstractmethod
    def types(self) -> Set[TimeType]:
        pass



from datetime import datetime
from typing import Set

from .day import DayTimeInfo
from .type import TimeType
from .week import WeekInfo


class TimeInfo(DayTimeInfo, WeekInfo):

    @property
    def types(self) -> Set[TimeType]:
        if not self._types:
            for info_type in TimeInfo.__bases__:
                self._types = self._types.union(info_type(self.date).types)

        return self._types


def whattime(date: datetime) -> TimeInfo:
    return TimeInfo(date)


def week_info(date: datetime) -> WeekInfo:
    return WeekInfo(date)


def day_time_info(date: datetime) -> DayTimeInfo:
    return DayTimeInfo(date)



from datetime import datetime
from typing import Set

from .base import InfoBase
from .type import TimeType


class WeekInfo(InfoBase):
    __mapping__ = {
        'is_weekend': TimeType.WEEKEND,
        'is_weekday': TimeType.WEEKDAY,
        'is_monday': TimeType.MONDAY,
        'is_tuesday': TimeType.TUESDAY,
        'is_wednesday': TimeType.WEDNESDAY,
        'is_thursday': TimeType.THURSDAY,
        'is_friday': TimeType.FRIDAY,
        'is_saturday': TimeType.SATURDAY,
        'is_sunday': TimeType.SUNDAY
    }

    @property
    def types(self) -> Set[TimeType]:
        """Return a set of fitting time types for the given datetime"""

        if self._types:
            return self._types

        for prop, time_type in self.__mapping__.items():
            predicate = self.__getattribute__(prop)

            if predicate:
                self._types.add(time_type)

        return self._types

    @property
    def is_weekend(self) -> bool:
        """Return whether the given datetime is a weekend day"""

        return self.is_saturday or self.is_sunday

    @property
    def is_weekday(self) -> bool:
        """Return whether the given datetime is a weekday"""

        return not self.is_weekend

    @property
    def is_monday(self) -> bool:
        """Return whether the given datetime is a Monday"""

        return self.date.isoweekday() == 1

    @property
    def is_tuesday(self) -> bool:
        """Return whether the given datetime is a Tuesday"""

        return self.date.isoweekday() == 2

    @property
    def is_wednesday(self) -> bool:
        """Return whether the given datetime is a Monday"""

        return self.date.isoweekday() == 3

    @property
    def is_thursday(self) -> bool:
        """Return whether the given datetime is a Thursday"""
        return self.date.isoweekday() == 4

    @property
    def is_friday(self) -> bool:
        """Return whether the given datetime is a Friday"""

        return self.date.isoweekday() == 5

    @property
    def is_saturday(self) -> bool:
        """Return whether the given datetime is a Saturday"""

        return self.date.isoweekday() == 6

    @property
    def is_sunday(self) -> bool:
        """Return whether the given datetime is a Sunday"""

        return self.date.isoweekday() == 7


def week_info(date: datetime) -> WeekInfo:
    return WeekInfo(date)



from enum import Enum


class TimeType(Enum):
    WEEKEND = 'weekend'
    WEEKDAY = 'weekday'

    MONDAY = 'monday'
    TUESDAY = 'tuesday'
    WEDNESDAY = 'wednesday'
    THURSDAY = 'thursday'
    FRIDAY = 'friday'
    SATURDAY = 'saturday'
    SUNDAY = 'sunday'

    MORNING = 'morning'
    MIDMORNING = 'midmorning'
    NOON = 'noon'
    AFTERNOON = 'afternoon'
    EVENING = 'evening'
    NIGHT = 'night'

    SPRINT = 'spring'
    SUMMER = 'summer'
    AUTUMN = 'autumn'
    WINTER = 'winter'



from datetime import datetime

from whattime import week_info, day_time_info

# Asking for week info only for a e.g. a monday afternoon:
now = datetime.now()
info = week_info(now)

print(info.types)
# {<TimeType.WEEKDAY: 'weekday'>, <TimeType.MONDAY: 'monday'>}

print(info.is_weekday)
# True

print(info.is_weekend)
# False

print(info.is_morning)
# AttributeError: 'WeekInfo' object has no attribute 'is_morning'


# Asking for day time info only:
info = day_time_info(now)

print(info.types)
# {<TimeType.AFTERNOON: 'afternoon'>}

print(info.is_afternoon)
# True

print(info.is_evening)
# False

print(info.is_weekend)
# AttributeError: 'DayTimeInfo' object has no attribute 'is_weekend'



from datetime import datetime

from whattime import TimeType
from whattime import whattime


def test_time_info(monday: datetime):
    """Test time_info combines multiple info"""

    monday_noon = monday.replace(hour=12, minute=30)
    info = whattime(monday_noon)

    assert info.types == {TimeType.MONDAY, TimeType.WEEKDAY, TimeType.NOON}

    assert info.is_monday is True
    assert info.is_tuesday is False
    assert info.is_wednesday is False
    assert info.is_thursday is False
    assert info.is_friday is False
    assert info.is_saturday is False
    assert info.is_sunday is False
    assert info.is_weekend is False
    assert info.is_weekday is True

    assert info.is_morning is False
    assert info.is_midmorning is False
    assert info.is_noon is True
    assert info.is_afternoon is False
    assert info.is_evening is False
    assert info.is_night is False



from datetime import datetime
from typing import Set

from .base import InfoBase
from .type import TimeType


class DayTimeInfo(InfoBase):
    __mapping__ = {
        range(0, 4): TimeType.NIGHT,
        range(5, 9): TimeType.MORNING,
        range(10, 11): TimeType.MIDMORNING,
        range(12, 13): TimeType.NOON,
        range(14, 18): TimeType.AFTERNOON,
        range(19, 22): TimeType.EVENING,
        range(23, 24): TimeType.NIGHT
    }

    __inverse_mapping__ = {v: k for k, v in __mapping__.items()}

    @property
    def types(self) -> Set[TimeType]:
        """Return a set of fitting time types for the given datetime"""

        if self._types:
            return self._types

        hour = self.date.hour

        for hours_range, time_type in self.__mapping__.items():
            if hour in hours_range:
                self._types.add(time_type)

        return self._types

    @property
    def is_morning(self) -> bool:
        """Return whether the given datetime is in the morning"""

        return self._is_in_daytime(TimeType.MORNING)

    @property
    def is_midmorning(self) -> bool:
        """Return whether the given datetime is in the midmorning"""

        return self._is_in_daytime(TimeType.MIDMORNING)

    @property
    def is_noon(self) -> bool:
        """Return whether the given datetime is at noon"""

        return self._is_in_daytime(TimeType.NOON)

    @property
    def is_afternoon(self) -> bool:
        """Return whether the given datetime is in the afternoon"""

        return self._is_in_daytime(TimeType.AFTERNOON)

    @property
    def is_evening(self) -> bool:
        """Return whether the given datetime is in the evening"""

        return self._is_in_daytime(TimeType.EVENING)

    @property
    def is_night(self) -> bool:
        """Return whether the given datetime is in the night"""

        return self._is_in_daytime(TimeType.NIGHT)

    def _is_in_daytime(self, time_type: TimeType) -> bool:
        hour_range = self.__inverse_mapping__[time_type]
        return isinstance(hour_range, range) and self.date.hour in hour_range


def day_time_info(date: datetime) -> DayTimeInfo:
    return DayTimeInfo(date)
